global string FW_DEVICE_COMMENT_CLASS; //Set to $sDeviceCommentClass in the panel init function

const string FW_DEVICE_COMMENT_DELIMITER = "~";

const unsigned FW_DEVICE_COMMENT_TABLE_COLUMN_DPE       = 0;
const unsigned FW_DEVICE_COMMENT_TABLE_COLUMN_DEVICE    = 1;
const unsigned FW_DEVICE_COMMENT_TABLE_COLUMN_TIMESTAMP = 2;
const unsigned FW_DEVICE_COMMENT_TABLE_COLUMN_USER      = 3;
const unsigned FW_DEVICE_COMMENT_TABLE_COLUMN_COMMENT   = 4;
const unsigned FW_DEVICE_COMMENT_TABLE_COLUMN_EXTENSION = 5;
const unsigned FW_DEVICE_COMMENT_TABLE_COLUMN_SYSTEM    = 6;
const unsigned FW_DEVICE_COMMENT_TABLE_COLUMN_HOST      = 7;
const unsigned FW_DEVICE_COMMENT_TABLE_COLUMN_ACTIVE    = 8;

const string FW_DEVICE_COMMENT_TABLE_REF = "fwDeviceComment_commentsTable";

//global string     gFwDeviceComment_getCommentsFunction;
global string     gFwDeviceComment_populateTableFunction;
global dyn_string gFwDeviceComment_tableChangedCallbackFunction;
global dyn_string gFwDeviceComment_filterAppliedCallbackFunction;
global dyn_string gFwDeviceComment_doubleClickCallbackFunction;
global unsigned   gFwDeviceComment_totalShown = 0;

public void fwDeviceComment_init() {
  fwDeviceFilter_applyFilter();
}

public void fwDeviceComment_addRow(string dpe, string device, time timestamp, string user, string comment, 
                                      string commentExt, string sys, string host, bool active) {  
  fwDeviceComment_commentsTable.updatesEnabled = false;
  fwDeviceComment_commentsTable.appendLine(
       "dpe", dpe,
       "device", device,
       "timestamp", timestamp,
       "user", user, 
       "comment", comment, 
       "extension", commentExt, 
       "system", sys, 
       "host", host,
       "active", active
   );
   fwDeviceComment_commentsTable.updatesEnabled = true;
}

public void fwDeviceComment_addRows(dyn_string dpe, dyn_string device, dyn_time timestamp, dyn_string user, dyn_string comment, 
                                      dyn_string commentExt, dyn_string sys, dyn_string host, dyn_bool active) {  
  fwDeviceComment_commentsTable.appendLines(
       dynlen(device),
       "dpe", dpe,
       "device", device,
       "timestamp", timestamp,
       "user", user, 
       "comment", comment, 
       "extension", commentExt, 
       "system", sys, 
       "host", host,
       "active", active
   );
}

public void fwDeviceComment_deleteAllRows() {
  fwDeviceComment_commentsTable.deleteAllLines();
}

/**
* Apply a filter to the table.
*  
* dpFilterList is the patterns of DPs to match for populating the table. 
* The filter is then applied after the table has been populated.
* The table will only be populated if hasDpFilterListChanged = true
* and it will only be filtered if hasChanged = true.
*  
* filterValues is a mapping { columnN => dyn_string(valuesToMatch) } or { columnN => "*" }
*/
public void fwDeviceComment_applyFilter(unsigned maxRows, bool hasDpFilterListChanged, bool hasChanged, dyn_string dpFilterList, 
                                          time start, time end, mapping filterValues) { 
   //firstly, repopulate the table if required
   if (hasDpFilterListChanged) {     
     fwDeviceComment_populateTable(maxRows, dpFilterList);      
   }    
   
   //then filter the table  
   if (hasChanged) {    
     gFwDeviceComment_totalShown = 0; 
     unsigned tableMax = fwDeviceComment_commentsTable.lineCount();
    
     //hide every row first
     for (unsigned rowN = 0; rowN < tableMax; rowN++) {
       fwDeviceComment_commentsTable.hideRow(rowN);
     }
      
     if (filterValues[(string)FW_DEVICE_COMMENT_TABLE_COLUMN_COMMENT] != "*") {
       //wrap comment filter in *s
       unsigned commentLength = dynlen(filterValues[(string)FW_DEVICE_COMMENT_TABLE_COLUMN_COMMENT]);
       for (unsigned i = 1; i <= commentLength; i++) {
         filterValues[(string)FW_DEVICE_COMMENT_TABLE_COLUMN_COMMENT][i]
             = "*" + filterValues[(string)FW_DEVICE_COMMENT_TABLE_COLUMN_COMMENT][i] + "*";
       }
     }
   
     unsigned filterLength = mappinglen(filterValues);     

     for (unsigned rowN = 0; rowN < tableMax; rowN++) {
       bool show = true;
       dyn_anytype row = fwDeviceComment_commentsTable.getLineN(rowN);
        
       if (!(row[FW_DEVICE_COMMENT_TABLE_COLUMN_TIMESTAMP + 1] >= start 
               && row[FW_DEVICE_COMMENT_TABLE_COLUMN_TIMESTAMP + 1] <= end)) {
         show = false;
       } else {             
         for (unsigned filterN = 1; filterN <= filterLength; filterN++) {
           unsigned columnN = mappingGetKey(filterValues, filterN);
           anytype  pattern = mappingGetValue(filterValues, (string)filterN);
         
           bool lShow = false;
           if (pattern == "*") {
             lShow = true;
           } else {
             unsigned dynLen = dynlen(pattern);
             string rowValue = row[columnN+1];
             
             //if it's the comment field then combine it with the comment extension field
             if (columnN == FW_DEVICE_COMMENT_TABLE_COLUMN_COMMENT) {
               rowValue = rowValue + row[FW_DEVICE_COMMENT_TABLE_COLUMN_EXTENSION + 1];
             }
             //check any of the filter values against the row value
             for (unsigned x = 1; x <= dynLen; x++) {
               if (patternMatch(pattern[x], rowValue)) {
                 lShow = true;
                 break;
               }
             }
           }
           show = show & lShow;
         }   
       }
       if (show) {
         gFwDeviceComment_totalShown++;
         fwDeviceComment_commentsTable.showRow(rowN);
       }
     }
   }
   
  unsigned callbackFunctionCount = dynlen(gFwDeviceComment_filterAppliedCallbackFunction);
  
  if (callbackFunctionCount > 0) {
    unsigned totalResults = fwDeviceComment_commentsTable.lineCount();  
    
    for (unsigned i = 1; i <= callbackFunctionCount; i++) {
      if (isFunctionDefined(gFwDeviceComment_filterAppliedCallbackFunction[i])) {
          execScript(
        			"main(unsigned totalResults, unsigned totalShown)" +
        			"{" +
        			"  " + gFwDeviceComment_filterAppliedCallbackFunction[i] + "(totalResults, totalShown);" +
        			"}",
        			makeDynString(),
           totalResults,
           gFwDeviceComment_totalShown
        		);        
      }
    }
  }  
}

public void fwDeviceComment_setPopulateTableFunction(string functionName) {
  if (!isFunctionDefined(functionName)) {
    //throw exception
  }
  gFwDeviceComment_populateTableFunction = functionName;
}

public void fwDeviceComment_populateTable(unsigned maxRows, dyn_string dpFilter = makeDynString()) {    
  if (isFunctionDefined(gFwDeviceComment_populateTableFunction)) {
    execScript(
  			"main(unsigned maxRows, dyn_string dpFilter)" +
  			"{" +
  			"  " + gFwDeviceComment_populateTableFunction + "(maxRows, dpFilter);" +
  			"}",
  			makeDynString(), 
     maxRows,
     dpFilter
  		);
  }
  unsigned callbackFunctionCount = dynlen(gFwDeviceComment_tableChangedCallbackFunction);

  if (callbackFunctionCount > 0) {
    for (unsigned i = 1; i <= callbackFunctionCount; i++) {
      if (isFunctionDefined(gFwDeviceComment_tableChangedCallbackFunction[i])) {
          execScript(
        			"main()" +
        			"{" +
        			"  " + gFwDeviceComment_tableChangedCallbackFunction[i] + "();" +
        			"}",
        			makeDynString()
        		);        
      }
    }
  } 
  gFwDeviceComment_totalShown = fwDeviceComment_commentsTable.lineCount(); 
}

public void fwDeviceComment_enable(const bool enable) {
  setValue("fwDeviceComment_commentsTable", "enabled", enable);
  setValue("importExportBtn", "enabled", enable);  
}

/**
* Get the total number of comments in the table.
*/
public unsigned fwDeviceComment_getTotal() {
  return fwDeviceComment_commentsTable.lineCount();
}

/**
* Get the total number of visible comments in the table (i.e. the number
* that have not been filtered).
*/
public unsigned fwDeviceComment_getTotalShown() {
  return gFwDeviceComment_totalShown;
}

/**
* Clear all the table change callbacks.
*/
public void fwDeviceComment_clearTableChangedCallbacks() {
  gFwDeviceComment_tableChangedCallbackFunction = makeDynString();
}

/**
* Add a table change callback - it will be executed when the table is repopulated.
*/
public void fwDeviceComment_addTableChangedCallback(string cbFunctionName) {
  if (!isFunctionDefined(cbFunctionName)) {
    //thow exception
  }
  dynAppend(gFwDeviceComment_tableChangedCallbackFunction, cbFunctionName);
}

/**
* Clear all the table change callbacks.
*/
public void fwDeviceComment_clearFilterAppliedCallbacks() {
  gFwDeviceComment_filterAppliedCallbackFunction = makeDynString();
}


/**
* Add a table change callback - it will be executed when the table is repopulated.
*/
public void fwDeviceComment_addFilterAppliedCallback(string cbFunctionName) {
  if (!isFunctionDefined(cbFunctionName)) {
    //thow exception
  }
  dynAppend(gFwDeviceComment_filterAppliedCallbackFunction, cbFunctionName);
}

/**
* Add a table change callback - it will be executed when the table is repopulated.
*/
public void fwDeviceComment_addDoubleClickCallback(string cbFunctionName) {
  if (!isFunctionDefined(cbFunctionName)) {
    //thow exception
  }
  dynAppend(gFwDeviceComment_doubleClickCallbackFunction, cbFunctionName);
}

public void fwDeviceComment_clearCallbacks() {
  dynClear(gFwDeviceComment_filterAppliedCallbackFunction);
  dynClear(gFwDeviceComment_tableChangedCallbackFunction);
  dynClear(gFwDeviceComment_doubleClickCallbackFunction);
  gFwDeviceComment_populateTableFunction = "";
}
