#uses "fwDevice/fwDeviceFilter.ctl"
#uses "fwDeviceComment/fwDeviceComment.ctl"
#uses "fwDeviceComment/fwDeviceCommentImportExport.ctl"

const string UN_DEVICE_COMMENT_DELIMITER = "~";

void unDeviceComment_init() {//pass in table ref here
  fwDeviceComment_clearCallbacks();
  fwDeviceFilter_clearCallbacks();
  
  fwDeviceFilter_addApplyFilterCallbackFunction("fwDeviceComment_applyFilter");       
  fwDeviceFilter_setInitialiseDpListFunction("unDeviceComment_initialiseDpList");
  fwDeviceComment_addTableChangedCallback("fwDeviceFilter_updateFilters");
  fwDeviceComment_addFilterAppliedCallback("fwDeviceFilter_setResultsTotal");  
  fwDeviceComment_addDoubleClickCallback("unDeviceComment_doubleClick");
    
  fwDeviceFilter_linkTableColumn(FW_DEVICE_COMMENT_TABLE_REF, FW_DEVICE_COMMENT_TABLE_COLUMN_DEVICE, FW_DEVICE_FILTER_LIST_1);
  fwDeviceFilter_linkTableColumn(FW_DEVICE_COMMENT_TABLE_REF, FW_DEVICE_COMMENT_TABLE_COLUMN_USER, FW_DEVICE_FILTER_LIST_2);
  fwDeviceFilter_linkTableColumn(FW_DEVICE_COMMENT_TABLE_REF, FW_DEVICE_COMMENT_TABLE_COLUMN_COMMENT, FW_DEVICE_FILTER_LIST_3, FW_DEVICE_FILTER_TYPE_TEXT);
  fwDeviceFilter_linkTableColumn(FW_DEVICE_COMMENT_TABLE_REF, FW_DEVICE_COMMENT_TABLE_COLUMN_SYSTEM, FW_DEVICE_FILTER_LIST_4);
  fwDeviceFilter_linkTableColumn(FW_DEVICE_COMMENT_TABLE_REF, FW_DEVICE_COMMENT_TABLE_COLUMN_HOST, FW_DEVICE_FILTER_LIST_5);
  fwDeviceFilter_linkTableColumn(FW_DEVICE_COMMENT_TABLE_REF, FW_DEVICE_COMMENT_TABLE_COLUMN_ACTIVE, FW_DEVICE_FILTER_LIST_6);           
  
  fwDeviceComment_setPopulateTableFunction("unDeviceComment_populateTable");
  
  fwDeviceComment_setImportFunction("unDeviceComment_import");
  fwDeviceComment_setExportFunction("unDeviceComment_export");
  fwDeviceFilter_setGroupQueryFunction("unDeviceFilter_setGroupQueryFunction");
  fwDeviceFilter_setFilterStartCallbackFunction("unDeviceComment_filterStartCB");
  fwDeviceFilter_setFilterEndCallbackFunction("unDeviceComment_filterEndCB");

  fwDeviceFilter_init(); 
  fwDeviceComment_init();
  
}

void unDeviceComment_filterStartCB() {
  fwOpenProgressBar("Searching for comments", "Searching...", 1);
  fwDeviceComment_enable(false);
}

void unDeviceComment_filterEndCB() {
  fwDeviceComment_enable(true);  
  fwCloseProgressBar("Search complete");
}

void unDeviceComment_doubleClick(dyn_string line) {
  string dpe        = line[FW_DEVICE_COMMENT_TABLE_COLUMN_DPE + 1];
  string deviceName = line[FW_DEVICE_COMMENT_TABLE_COLUMN_DEVICE + 1];
  ChildPanelOnCentral(
    "vision/unComment/unDeviceComment.pnl",
    "Comments for " + deviceName, 
    makeDynString(
      "$sDeviceDpe:" + dpe,
      "$sDeviceName:" + deviceName
    )
  );
}

void unDeviceComment_populateTable(unsigned max, const dyn_string dpList = makeDynString()) {
    fwDeviceComment_deleteAllRows();
    dyn_string dpe, device;
    dyn_time timestamp;
    dyn_string user, comment;
    dyn_string commentExt, sys, host;
    dyn_bool active;
    
    unsigned total = 0;
    for (unsigned i = 1; i <= dynlen(dpList); i++) {
      dyn_string dpes = dpNames(dpList[i] + ".statusInformation.deviceLog");

      for (unsigned i = 1; i <= dynlen(dpes); i++) {
        if (dpExists(dpes[i] + "Active")) {
          dyn_string deviceLog;
          bool deviceLogActive;
          dpGet(dpes[i], deviceLog);
          if (dynlen(deviceLog) > 0) {
              string deviceName = dpSubStr(dpes[i], DPSUB_SYS_DP);
              string alias      = unGenericDpFunctions_getAlias(deviceName);
              unsigned length   = dynlen(deviceLog);
            
              for (unsigned i = 1; i <= length; i++) {
                    dyn_string commentRow = unDeviceComment_decodeCommentString(deviceLog[i]);
                    
                    if (total++ >= max) {
                        fwDeviceComment_addRows(dpe, device, timestamp, user, comment, commentExt, sys, host, active);
                      return;
                    }
                    dynAppend(dpe, deviceName);
                    dynAppend(device, alias);
                    dynAppend(timestamp, commentRow[1]);
                    dynAppend(user, commentRow[2]);
                    dynAppend(comment, commentRow[3]);
                    dynAppend(commentExt, commentRow[4]);
                    dynAppend(sys, commentRow[5]);
                    dynAppend(host, commentRow[6]);
                    dynAppend(active, (bool)commentRow[7]);
              }            
          }
        }
      }   
    }

    fwDeviceComment_addRows(dpe, device, timestamp, user, comment, commentExt, sys, host, active);    
}

dyn_string unDeviceComment_initialiseDpList() {
  return makeDynString("*");
}

string unDeviceFilter_setGroupQueryFunction(string sys, string queryFrom, string queryWhere) {
    string query = "SELECT \'_original.._value\' ";
    
    query = query + queryFrom;
    
    query = query + "REMOTE \'" + sys + "\' ";
          
    query = query + " WHERE _EL = \"statusInformation.deviceLog\" ";
    
    if (queryWhere != "") {
      query = query + queryWhere;
    }   
      
    return query;
}

bool unDeviceComment_import(const string filename) {
    file inputFile = fopen(filename, "r");
    while (feof(inputFile) == 0) {
        string currentLine;
        fgets(currentLine, 100000, inputFile);
        if (currentLine == "") continue;
        int idx;
        idx = strpos(currentLine, ";");
        string alias = substr(currentLine, 0, idx);
        currentLine = substr(currentLine, idx+1);
        
        idx = strpos(currentLine, ";");
        string deviceLogActive_s = substr(currentLine, 0, idx);
        bool deviceLogActive = strtolower(strrtrim(strltrim(deviceLogActive_s))) == "true";
        currentLine = substr(currentLine, idx+1);
        currentLine = strrtrim(currentLine, "\n");
        dyn_string deviceLog = strsplit(currentLine, ";");

        string deviceName = unGenericDpFunctions_dpAliasToName(alias);

        dpSetWait(deviceName + "statusInformation.deviceLogActive", deviceLogActive,
                  deviceName + "statusInformation.deviceLog", deviceLog);

    }
    fclose(inputFile);
    return true;
}

bool unDeviceComment_export(const string filename) {
    file outputFile = fopen(filename, "w");

    dyn_string dpes = dpNames(getSystemName() + "*.statusInformation.deviceLog");
    for (int i = 1; i <= dynlen(dpes); i++) {
        dyn_string deviceLog;
        bool deviceLogActive;
        dpGet(dpes[i], deviceLog, dpes[i] + "Active", deviceLogActive);
        if (dynlen(deviceLog) > 0) {
            string deviceName = dpSubStr(dpes[i], DPSUB_SYS_DP);
            string alias = unGenericDpFunctions_getAlias(deviceName);
            fputs(alias + ";" + deviceLogActive + ";" + escape(deviceLog[1]), outputFile);
            for (int i = 2; i <= dynlen(deviceLog); i++) {
                fputs(";" + escape(deviceLog[i]), outputFile);
            }
            fputs("\n", outputFile);
        }
    }
    fclose(outputFile);
    return true;
}


private string escape(string text) {
    strreplace(text, ";", ",");
    return text;
}

public dyn_string unDeviceComment_decodeCommentString(string s) {
  dyn_string result = strsplit(s, UN_DEVICE_COMMENT_DELIMITER);
  for (int i = 1; i <= dynlen(result); i++) {
      strreplace(result[i], ":71LD3:", UN_DEVICE_COMMENT_DELIMITER);
  }
  return result;
}

public string unDeviceComment_encodeCommentArray(dyn_string r) {
  string s = "";
  int len = dynlen(r);
  for (int i = 1; i <= len; i++) {
    string str = r[i];
    strreplace(str, UN_DEVICE_COMMENT_DELIMITER, ":71LD3:");
    s = s + str;
    if (i != len) {
      s = s + UN_DEVICE_COMMENT_DELIMITER;
    }
  }
  return s;
}





//TODO
/*

void unDeviceComment_addTestComments(const dyn_string dpList) {
  for (unsigned i = 1; i <= dynlen(dpList); i++) {
      dyn_string dpes = dpNames(dpList[i]);

      for (unsigned i = 1; i <= dynlen(dpes); i++) {
          int n = rand() % 10;
          Debug("creating comment " + n + " comments for " + dpes[i]);
          for (int j = 0; j < n; j++) {
            unDeviceComment_saveNewComment(dpes[i], "test comment " + j, "extended test comment " + j, true);
          }
      }      
  }
}


const string TIME_FORMAT = "%Y.%m.%d %H:%M:%S";
const string DEFAULT_COMMENT_TEXT = "Type your comment ...";
const string DEFAULT_EXTENSION_TEXT = "Add additional information ...";

const int ERROR_CODE_NO_ERROR = 0;
const int ERROR_CODE_GET_DP_ERROR = -1;
const int ERROR_CODE_SET_DP_ERROR = -2;
const int ERROR_CODE_NO_CONFIRMATION = -3;
const int ERROR_CODE_NO_COMMENT = -4;

// Add 1 to make it an index
const int USER_NAME_COLUMN = 3;
const int TIMESTAMP_COLUMN = 0;
const int ACTIVE_STATE_COLUMN = 6;

const int IS_ACTIVE = 1;

const int MAX = 10;


string getFullUserName(string sUserName = "")
{
  int iUserId;
  bool bEnabled;
  string sComment, sLogMessage, sFullUserName, sDescription;
  dyn_string dsGroupNames, exceptionInfo;

  if("" == sUserName)
  {
    fwAccessControl_getUserName(sUserName);
  }
  
  fwAccessControl_getUser(sUserName, sFullUserName, sDescription, iUserId, bEnabled, dsGroupNames, exceptionInfo);
  
  return sFullUserName;
}




private void unDeviceComment_saveNewComment(string dpe, string sCommentText, string sExtensionText, bool bActiveState = true)
{
  //dpSetWait(dpe, makeDynString(), dpe + "Active", false);
  //return;
  string sCurrentTime;
  string sSystemName, sHostName, sIPAddress, sUserName, sLogMessage, sComment;
  dyn_string dsStoredComments;

  sCurrentTime = formatTime(TIME_FORMAT, getCurrentTime());
  sSystemName = getSystemName();
  unGenericDpFunctions_getHostName(sSystemName, sHostName, sIPAddress);
  fwAccessControl_getUserName(sUserName);

  sLogMessage = getFullUserName(sUserName) + " saved a new comment on " + dpe + ".";
   
  if(sExtensionText == DEFAULT_EXTENSION_TEXT)
  {
    sExtensionText = "";
  }
  
  sComment = (string) sCurrentTime + UN_DEVICE_COMMENT_DELIMITER
             + sUserName + UN_DEVICE_COMMENT_DELIMITER
             + sCommentText + UN_DEVICE_COMMENT_DELIMITER
             + sExtensionText + UN_DEVICE_COMMENT_DELIMITER
             + sSystemName + UN_DEVICE_COMMENT_DELIMITER
             + sHostName + UN_DEVICE_COMMENT_DELIMITER                          
             + (string) bActiveState;
  
  dsStoredComments = getCommentsFromDp(dpe);
  
  dynInsertAt(dsStoredComments, sComment, 1);
  
  // we only store as many comments as defined in UN_DEVICE_MAXIMUM_COMMENTS
  while(dynlen(dsStoredComments) > MAX)
  {
    dynRemove(dsStoredComments, dynlen(dsStoredComments));
  }
  
  dpSetWait(dpe, dsStoredComments, dpe + "Active", hasActiveComment(dsStoredComments));
}
*/
/** Checks whether list of comments has at least one active comment

@par Constraints
  None
  
@par Usage
	Private to the panel

@par WinCC managers
	VISION
  
@param dsStoredComments       dyn_string, INPUT, the prepared comments to be written

@return bool, active or not
*//*
private bool hasActiveComment(dyn_string dsStoredComments)
{
    dyn_string dsCommentSections;

    for(int index = 1; index <= dynlen(dsStoredComments); index++) 
    {
        dsCommentSections = strsplit(dsStoredComments[index], UN_DEVICE_COMMENT_DELIMITER);
        if(dsCommentSections[ACTIVE_STATE_COLUMN+1] == TRUE)
        {
            return TRUE;
        }
    }
    return FALSE;
}


*/
/** Getter: Retrieves the comments from the datapoint.

@par Constraints
  None
  
@par Usage
	Private to the panel

@par WinCC managers
	VISION
  
@return dyn_string, the prepared comments as array
*//*
dyn_string getCommentsFromDp(string dpe)
{
  int iReturnCode;
  dyn_string dsStoredComments;
  
  iReturnCode = dpGet(dpe, dsStoredComments);
  
  return dsStoredComments;
}
*/
